import http from 'node:http';
import { Socket } from 'node:net';
import { EggApplicationCore, type EggApplicationCoreOptions, type Context } from './egg.js';
import { AppWorkerLoader } from './loader/index.js';
import Helper from '../app/extend/helper.js';
declare const EGG_LOADER: unique symbol;
/**
 * Singleton instance in App Worker, extend {@link EggApplicationCore}
 * @augments EggApplicationCore
 */
export declare class Application extends EggApplicationCore {
    #private;
    server?: http.Server;
    /**
     * reference to {@link Helper}
     * @member {Helper} Application#Helper
     */
    Helper: typeof Helper;
    /**
     * @class
     * @param {Object} options - see {@link EggApplicationCore}
     */
    constructor(options?: Omit<EggApplicationCoreOptions, 'type'>);
    protected load(): Promise<void>;
    get [EGG_LOADER](): typeof AppWorkerLoader;
    onClientError(err: any, socket: Socket): void;
    onServer(server: http.Server): void;
    /**
     * global locals for view
     * @member {Object} Application#locals
     * @see Context#locals
     */
    get locals(): Record<string, any>;
    set locals(val: Record<string, any>);
    /**
     * save routers to `run/router.json`
     * @private
     */
    dumpConfig(): void;
    /**
     * Run async function in the background
     * @see Context#runInBackground
     * @param {Function} scope - the first args is an anonymous ctx
     */
    runInBackground(scope: (ctx: Context) => Promise<void>, req?: unknown): void;
    /**
     * secret key for Application
     * @member {String} Application#keys
     */
    get keys(): string[];
    /**
     * @deprecated keep compatible with egg 3.x
     */
    toAsyncFunction(fn: (...args: any[]) => any): (...args: any[]) => any;
}
declare module '@eggjs/core' {
    interface EggCore {
        onClientError(err: any, socket: Socket): void;
        onServer(server: http.Server): void;
        locals: Record<string, any>;
        runInBackground(scope: (ctx: Context) => Promise<void>, req?: unknown): void;
        toAsyncFunction(fn: (...args: any[]) => any): (...args: any[]) => any;
        dumpConfig(): void;
        get keys(): string[];
    }
}
export {};
